# -*- coding: utf-8 -*-
'''
  To  analyse stock ,basic valuation information
  聚宽的季度数据是累计到报告期为止的数据，进行滚动分析时，使用的季度数据
  应当是当期发生的数据,从最近一期往前面计算，最后一期在最下面，因此是从最下面一行开始
      本期发生 = 累计到本期-累计到上期
      一季度数据不用扣减，其他季度如何正好排在第一行，这个季度的实际发生值，将无法计算。
'''
import os
import numpy as np
import pandas as pd
import datetime
from jqdatasdk import *
acc_jq = os.environ["acc_jq"]
passwd_jq = os.environ["passwd_jq"]

auth(acc_jq,passwd_jq)

def get_info(code=None,type=None):
    '''
    get basic info : market_cap,net_profit
    code company symbol in stock market
    type = year or quarter
    the order can't changed:report_date,total_assets,total_owners_equities,
    operating_revenue,net_profit,np_parent_company_owners,
    total_assets_parent,total_owners_equities_parent,
    operating_revenue_parent,net_profit_parent,np_parent_company_owners_parent,
    '''
    query_industry = query(finance.STK_COMPANY_INFO.industry_id).\
                           filter(finance.STK_COMPANY_INFO.code==code)
    industry = finance.run_query(query_industry).iloc[0,0]
    financeIndustry = ["J66","J67","J68","J69"]
    # fiance and other company has different type table
    if ( industry in  financeIndustry ) :
        query_balance = query(finance.FINANCE_BALANCE_SHEET.report_date,
                            finance.FINANCE_BALANCE_SHEET.total_assets,
                            finance.FINANCE_BALANCE_SHEET.total_owner_equities
                            ).filter(
                            finance.FINANCE_BALANCE_SHEET.code == code,
                            finance.FINANCE_BALANCE_SHEET.report_type== 0)
        df_balance = finance.run_query(query_balance)
        query_income = query(finance.FINANCE_INCOME_STATEMENT.report_date,
                   finance.FINANCE_INCOME_STATEMENT.operating_revenue,
                   finance.FINANCE_INCOME_STATEMENT.net_profit,
                   finance.FINANCE_INCOME_STATEMENT.np_parent_company_owners
                   ).filter(finance.FINANCE_INCOME_STATEMENT.code == code,
                   finance.FINANCE_INCOME_STATEMENT.report_type==0)
        df_income = finance.run_query(query_income)
        query_balance_parent = query(finance.FINANCE_BALANCE_SHEET_PARENT.report_date,
                            finance.FINANCE_BALANCE_SHEET_PARENT.total_assets,
                            finance.FINANCE_BALANCE_SHEET_PARENT.total_owner_equities
                            ).filter(
                            finance.FINANCE_BALANCE_SHEET_PARENT.code == code,
                            finance.FINANCE_BALANCE_SHEET_PARENT.report_type== 0)
        df_balance_parent = finance.run_query(query_balance_parent)
        query_income_parent = query(finance.FINANCE_INCOME_STATEMENT_PARENT.report_date,
                   finance.FINANCE_INCOME_STATEMENT_PARENT.operating_revenue,
                   finance.FINANCE_INCOME_STATEMENT_PARENT.net_profit,
                   finance.FINANCE_INCOME_STATEMENT_PARENT.np_parent_company_owners
                   ).filter(finance.FINANCE_INCOME_STATEMENT_PARENT.code == code,
                   finance.FINANCE_INCOME_STATEMENT_PARENT.report_type==0)
        df_income_parent = finance.run_query(query_income)
        df = df_balance.join(df_income.set_index('report_date'),\
                             on='report_date',how='outer')
        df_parent = df_balance_parent.join(df_income_parent.set_index('report_date'),\
                                           on='report_date',how='outer')
        df_parent.rename(columns={'total_assets':'total_assets_parent',
                                  'total_owner_equities':'total_owner_equities_parent',
                                  'operating_revenue':'operating_revenue_parent',
                                  'net_profit':'net_profit_parent',
                                  'np_parent_company_owners':\
                                  'np_parent_company_owners_parent',
                                  },inplace=True)
    else :
        query_balance = query(finance.STK_BALANCE_SHEET.report_date,
                            finance.STK_BALANCE_SHEET.total_assets,
                            finance.STK_BALANCE_SHEET.total_owner_equities
                            ).filter(
                            finance.STK_BALANCE_SHEET.code == code,
                            finance.STK_BALANCE_SHEET.report_type== 0)
        df_balance = finance.run_query(query_balance)
        query_income = query(finance.STK_INCOME_STATEMENT.report_date,
                   finance.STK_INCOME_STATEMENT.total_operating_revenue,
                   finance.STK_INCOME_STATEMENT.net_profit,
                   finance.STK_INCOME_STATEMENT.np_parent_company_owners
                   ).filter(finance.STK_INCOME_STATEMENT.code == code,
                   finance.STK_INCOME_STATEMENT.report_type==0)
        df_income = finance.run_query(query_income)
        query_balance_parent = query(finance.STK_BALANCE_SHEET_PARENT.report_date,
                            finance.STK_BALANCE_SHEET_PARENT.total_assets,
                            finance.STK_BALANCE_SHEET_PARENT.total_owner_equities
                            ).filter(
                            finance.STK_BALANCE_SHEET_PARENT.code == code,
                            finance.STK_BALANCE_SHEET_PARENT.report_type== 0)
        df_balance_parent = finance.run_query(query_balance_parent)
        query_income_parent = query(finance.STK_INCOME_STATEMENT_PARENT.report_date,
                   finance.STK_INCOME_STATEMENT_PARENT.total_operating_revenue,
                   finance.STK_INCOME_STATEMENT_PARENT.net_profit,
                   finance.STK_INCOME_STATEMENT_PARENT.np_parent_company_owners
                   ).filter(finance.STK_INCOME_STATEMENT_PARENT.code == code,
                   finance.STK_INCOME_STATEMENT_PARENT.report_type==0)
        df_income_parent = finance.run_query(query_income_parent)
        df = df_balance.join(df_income.set_index('report_date'),\
                             on='report_date',how='outer')
        df_parent = df_balance_parent.join(df_income_parent.set_index('report_date'),\
                                           on='report_date',how='outer')
        df_parent.rename(columns=
                {'total_assets':'total_assets_parent',
                  'total_owner_equities':'total_owner_equities_parent',
                  'total_operating_revenue':'total_operating_revenue_parent',
                  'net_profit':'net_profit_parent',
                  'np_parent_company_owners':'np_parent_company_owners_parent',
                },inplace=True)
    df = df.join(df_parent.set_index('report_date'),on='report_date',how='outer')
    df = df.sort_values(by='report_date')
    df.reset_index(drop=True, inplace=True)
    all_cap = []
    query_market_cap = query(valuation.market_cap).filter(valuation.code == code) 
    for idate in df['report_date']:
        cap = get_fundamentals(query_market_cap,date=idate)['market_cap']
        if cap.empty:
            all_cap.append(np.nan)
        else:
            all_cap.append(cap[0])
    df['market_cap'] = all_cap
    df.index = df['report_date']
    #必须深度拷贝，后面的运算将修改df,不深度拷贝，raw_df也会变化
    raw_df = df.copy(deep=True)

    if type == "year" :
        df = df[df['report_date'].astype(str).str.contains('12-31')]
        return {"raw":raw_df,"peryear":df}
    elif type == "quarter" :
        row_num = df.shape[0]
        col_num = df.shape[1]
        #对这个df从最后一行往前面进行，因为后面的一行，根据上面一行修改
        #必须先改后面的每行，改完后，固定下来
        for row_id in range(row_num-1,-1,-1):
            if df.iloc[row_id,0].strftime('%Y-%m-%d')[5:]  != '03-31':
                if (row_id - 1 >= 0 and df.iloc[row_id-1,0].strftime('%Y') == \
                                  df.iloc[row_id,0].strftime('%Y')):
                    df.iloc[row_id,3] = df.iloc[row_id,3] - df.iloc[row_id-1,3]
                    df.iloc[row_id,4] = df.iloc[row_id,4] - df.iloc[row_id-1,4]
                    df.iloc[row_id,5] = df.iloc[row_id,5] - df.iloc[row_id-1,5]
                    df.iloc[row_id,8] = df.iloc[row_id,8] - df.iloc[row_id-1,8]
                    df.iloc[row_id,9] = df.iloc[row_id,9] - df.iloc[row_id-1,9]
                    df.iloc[row_id,10] = df.iloc[row_id,10] - df.iloc[row_id-1,10]
                elif row_id == 0:
                    if df.iloc[row_id,0].strftime('%m') == '06':
                        df.iloc[row_id,3] = df.iloc[row_id,3]/2 
                        df.iloc[row_id,4] = df.iloc[row_id,4]/2
                        df.iloc[row_id,5] = df.iloc[row_id,5]/2
                        df.iloc[row_id,8] = df.iloc[row_id,8]/2
                        df.iloc[row_id,9] = df.iloc[row_id,9]/2
                        df.iloc[row_id,10] = df.iloc[row_id,10]/2
                    if df.iloc[row_id,0].strftime('%m') == '09':
                        df.iloc[row_id,3] = df.iloc[row_id,3]/3
                        df.iloc[row_id,4] = df.iloc[row_id,4]/3
                        df.iloc[row_id,5] = df.iloc[row_id,5]/3
                        df.iloc[row_id,8] = df.iloc[row_id,8]/3
                        df.iloc[row_id,9] = df.iloc[row_id,9]/3
                        df.iloc[row_id,10] = df.iloc[row_id,10]/3
                    if df.iloc[row_id,0].strftime('%m') == '12':
                        df.iloc[row_id,3] = df.iloc[row_id,3]/4
                        df.iloc[row_id,4] = df.iloc[row_id,4]/4
                        df.iloc[row_id,5] = df.iloc[row_id,5]/4
                        df.iloc[row_id,8] = df.iloc[row_id,8]/4
                        df.iloc[row_id,9] = df.iloc[row_id,9]/4
                        df.iloc[row_id,10] = df.iloc[row_id,10]/4
        return {"raw":raw_df,"perquarter":df}



# -*- coding: utf-8 -*-
'''
求出波动性，并绘图,本地没有数据，就从yahoo上抓数据
dp means date and price
example:
    import analyse.data.vix as vis
    stock = ""
    vix.draw_vix(data=df,stock='somestock')
    vix.draw_vix(stock='',num=500)
    vix.draw_vix(stock,num,market='sz')  
    vix.draw_vix(stock,num,market='sh')

'''
import os,io
import re,time,datetime
import http.cookiejar
import urllib.request
import pymysql
import pandas as pd
from pandas.plotting import register_matplotlib_converters
register_matplotlib_converters()
import matplotlib.pyplot as plt
from pylab import *
from sqlalchemy import create_engine

base_dir = '/tmp'
user = 'root'
mysql_ip = '127.0.0.1'
mysql_pass = os.environ['mysql_pass']

hkengine = create_engine('mysql+pymysql://root:{}@{}:3306/hksec?charset=utf8'.\
                          format(mysql_pass,mysql_ip))
szengine = create_engine('mysql+pymysql://root:{}@{}:3306/szsec?charset=utf8'.\
                          format(mysql_pass,mysql_ip))
shengine = create_engine('mysql+pymysql://root:{}@{}:3306/shsec?charset=utf8'.\
                          format(mysql_pass,mysql_ip))

mproperty = {'hk':{'engine':hkengine},
             'sz':{'engine':szengine},
             'sh':{'engine':shengine}}

def get_ohlc_yahoo(stock='^hsi',num=None):
    ''' get data in `open high low close` format ,reindex from 0
    normal stock : get_ohlc_yahoo(stock='msft',num=None)
    index        : get_ohlc_yahoo(stock='^dji',num=None)  
                   get_ohlc_yahoo(stock='^hsi',num=None)
    x=get_ohlc_yahoo(stock='^hsi',num=500)
    '''
    start=0
    end  = int(time.time())
    target1 = "https://finance.yahoo.com/quote/{}/history?p={}".format(stock,stock)
    fname_cookie = os.path.join(base_dir,'cookies.txt')
    cookie = http.cookiejar.MozillaCookieJar(fname_cookie)
    handler = urllib.request.HTTPCookieProcessor(cookie)
    opener = urllib.request.build_opener(handler)
    response1 = opener.open(target1).read().decode('utf-8')
    cookie.save()
    crumb = re.findall('"CrumbStore":{"crumb":"(.+?)"}', response1)[0] 
    cookie.load(fname_cookie)
    handler = urllib.request.HTTPCookieProcessor(cookie)
    opener = urllib.request.build_opener(handler)
    time.sleep(3)
    target2 = "https://query1.finance.yahoo.com/v7/finance/download/\
               {0}?period1={1}&period2={2}&interval=1d&events=history&crumb={3}".\
               format(stock,start,end,crumb)
    response2 = opener.open(target2).read().decode('utf-8')
    df = pd.read_csv(io.StringIO(response2),sep=',')
    tmpdf = df.dropna(axis=0,how='any')
    tmpdf.columns = ['date','open','high','low','close','amount','volume']
    newdf = pd.DataFrame(columns = ['code','date','open','high','low',\
                                    'close','amount','volume'] )
    code = stock.replace('^','')
    for id in range(0,len(tmpdf)):
        newdf.loc[id] = [code,tmpdf.iloc[id]['date'],round(tmpdf.iloc[id]['open'],2),    \
                     round(tmpdf.iloc[id]['high'],2),round(tmpdf.iloc[id]['low'],2),     \
                     round(tmpdf.iloc[id]['close'],2),0,int(tmpdf.iloc[id]['volume'])]
    os.remove(fname_cookie)
    if num is None:
        newdf = newdf
    else:
        newdf = newdf.iloc[-num:,]
    newdf.index = range(len(newdf))
    return newdf



def get_dp_yahoo(stock='^hsi',num=500):
    '''get data in `date price` format from ohlc 
    normal stock : get_dp_yahoo(stock='msft',num=None)
    index        : get_dp_yahoo(stock='^dji',num=None)  
                   get_dp_yahoo(stock='^hsi',num=None)
    '''
    df = get_ohlc_yahoo(stock=stock,num=num)
    date = df['date']
    price = df['close']
    newdf = pd.DataFrame({'date':date,'price':price})
    return newdf

def get_ohlc_local(stock=None,num=None,market=None):
    stock = stock.replace('^','')
    if num is None:
        sql_code = 'SELECT date,open,high,low,close FROM quote where code="{}";'\
                    .format(stock)
    else:
        sql_code = 'select * from (select date,open,high,low,close from quote \
                    where code="{}" order by date desc limit {}) as tmp order \
                    by date asc;'.format(stock,num) 
    try:
        df = pd.read_sql(sql_code, con=mproperty[market]['engine']) 
    except:
        return None
    return  df   


def get_dp_local(stock=None,num=None,market=None):
    '''
        get_dp_local(stock,market='sz')  get_dp_local(stock,market='sh')
    '''
    stock = stock.replace('^','')
    if num is None:
        sql_code = 'SELECT date,close as price FROM quote where code="{}";'\
                    .format(stock)
    else:
        sql_code = 'select * from (select date,close as price from quote \
                    where code="{}" order by date desc limit {}) as tmp  \
                    order by date asc;'.format(stock,num) 
    try:
        df = pd.read_sql(sql_code, con=mproperty[market]['engine']) 
    except:
        return None
    return  df   


def get_local_max_min(data=None,stock=None,num=None,market=None):
    '''data is None ,get_local_max_min from stcok and num;
       list to reindex from 0 and get data for using   
    normal stock : get_local_max_min(stock='msft',num=None)
    index        : get_local_max_min(stock='^dji',num=None)  
                   get_local_max_min(stock='^hsi',num=None)
                   get_local_max_min(data,stock,market='sz')  
                   get_local_max_min(data,stock,market='sh')

    '''
    if(data is None):
        code = stock.replace('^','')
        result = get_dp_local(stock=code,num=num,market=market)
        if result is not None:
            df = result
        else:
            df = get_dp_yahoo(stock=stock,num=num)
    else:
        df = data
    price = list(df['price'])
    date = list(df['date'])
    number = len(df)
    local_max_min = []
    if number<21 :
        print("the stock {}  's data is too small  ")
        return
    end_id = number - 1
    id=10
    while id<= end_id-11:
        flag1 = max(price[id-10:id])
        flag2 = min(price[id+1:id+11])
        flag3 = min(price[id-10:id])
        flag4 = max(price[id+1:id+11])
        if ( price[id] >= flag1 and price[id] >= flag4 ):
            local_max_min.append([date[id],price[id],'lmax'])
            id = id +10
        elif( price[id] <= flag2 and price[id] <= flag3 ):
            local_max_min.append([date[id],price[id],'lmin'])
            id = id +10
        else:
            id = id +1
    if local_max_min :
        return local_max_min
    else:
        print('no local max or min in the period')
        return None


def draw_vix(data=None,stock=None,num=None,market=None):
    '''draw vix ,when no local max or min ,return None ,draw nothing
       input data contains date and price to draw
       when data is not None,you have to input code='' in function 
       as the following two formats:
       draw_vix(data=df,stock='somestock')
       draw_vix(stock='',num=500)
       draw_vix(stock,num,market='sz')  
       draw_vix(stock,num,market='sh')
    normal stock : draw_vix(stock='msft',num=None)
    index        : draw_vix(stock='^dji',num=None)  draw_vix(stock='^hsi',num=None)      
    '''
    if data is None :
        code = stock.replace('^','')
        result = get_dp_local(stock=code,num=num,market=market)
        if result is not None:
            df = result 
        else:
            df = get_dp_yahoo(stock=stock,num=num)
            code = stock
    else:
        #recombine it as new df,change index from 0
        #changed index in getdata,how about if accept data 
        #with index not beginning from 0 ?
        newdate = list(data['date'])
        newprice = list(data['price'])
        df = pd.DataFrame({'date':newdate,'price':newprice})
    local_max_min = get_local_max_min(data=df,stock=stock,num=num)
    if local_max_min is not None:
        plt.ion()
        date = df['date']
        price = df['price']
        figure, ax = plt.subplots()
        xticks = list(range(0,len(date),10))
        if( xticks[-1] != len(date) ): xticks.append(len(date)-1)
        xlabels = date[xticks]
        ax.set_xticks(xticks)
        ax.set_xticklabels(xlabels,rotation=90)
        ax.plot(date,price)
        for item in local_max_min:
            ax.plot(item[0],item[1],'x',color='red')
        ax.set_title("{}'s vix in {} days".format(code,len(df)),fontsize=12,color='r')
        plt.draw()
        return  {'data':df,'local_max_min':local_max_min}
    else:
        print("no local max and min in the period,can't draw")
        return None



# -*- coding: utf-8 -*-
'''
get and write shanghai and shengzheng code into database 
'''
import time,os,sys,datetime
import pandas as pd
from sqlalchemy import create_engine
from jqdatasdk import *
import jqdatasdk

mysql_pass = os.environ['mysql_pass']
mysql_ip = '127.0.0.1'
acc_jq = os.environ['acc_jq']
passwd_jq = os.environ['passwd_jq']
user = 'root'

sz_engine = create_engine("mysql+pymysql://{}:{}@{}:3306/szsec?charset=utf8".\
                           format(user,mysql_pass,mysql_ip))
sh_engine = create_engine("mysql+pymysql://{}:{}@{}:3306/shsec?charset=utf8".\
                           format(user,mysql_pass,mysql_ip))


def get_latest(target):
    ''' get_all_trade_days create all trades day in a year,\
        getlast get the latest trade day forword'''
    xday = get_all_trade_days()
    latest = target + datetime.timedelta(-1)
    while (latest not in xday):
        latest = latest + datetime.timedelta(-1)
    return latest


def get_code():
    '''get sz,sh code from jqdatasdk;get hk code from hkex'''
    try:
        auth(acc_jq, passwd_jq)
        today = datetime.date.today()
        latest = get_latest(today)
        tmp_stock = get_all_securities(types=['stock'],date=latest)
        sz_stock = pd.DataFrame(columns=['code','name'])
        sh_stock = pd.DataFrame(columns=['code','name'])
        for row in tmp_stock.iterrows():
            if(row[0][0] != '6'):
                sz_stock = sz_stock.append({'code':row[0][0:6] ,'name':row[1][0]},\
                           ignore_index=True)
            else:
                sh_stock = sh_stock.append({'code':row[0][0:6] ,'name':row[1][0]},\
                           ignore_index=True)
        #add 510050 both into szsec.code and shsec.code
        sz_stock = sz_stock.append({'code':'510050' ,'name':'50etf'},ignore_index=True)  
        sh_stock = sh_stock.append({'code':'510050' ,'name':'50etf'},ignore_index=True)      
    except:
        sz_stock = None
        sh_stock = None
    return  (sz_stock,sh_stock)


def write_code():
    '''
    write code into database , szsec and shsec
    '''
    sz_stock,sh_stock = get_code()
    if sz_stock is not None and sh_stock is not None:
        sz_stock.to_sql('code',sz_engine,if_exists='replace', index=False)
        sh_stock.to_sql('code',sh_engine,if_exists='replace', index=False)
        return (sz_stock['code'],sh_stock['code'])
    else:
        return None







# -*- coding: utf-8 -*-
'''
  To  analyse stock ,basic information
'''
import analyse
import os
import numpy as np
import pandas as pd
import datetime
from jqdatasdk import *
acc_jq = os.environ["acc_jq"]
passwd_jq = os.environ["passwd_jq"]

auth(acc_jq,passwd_jq)

def rounddf(df):
    df_print = df.copy(deep=True)
    collen = df.shape[1]
    for item in range(collen):
        if df.iloc[:,item].dtype == 'float64':
            df.iloc[:,item] = df.iloc[:,item].apply(lambda x : round(x,4) \
                    if np.isnan(x) == False  else x)
            df_print.iloc[:,item] = df_print.iloc[:,item].apply(lambda x : \
                    str(round(x*100,2)) + '%' if np.isnan(x) == False  else x)
    return {'raw':df,'print':df_print}


def get_growth(code=None,type=None):
    '''
    get_growth(code=None,type=None) code contain surfix,type :
       year,quarter(当期的数值，非累计)
    return a dictionary,  {'raw':data,'growth':df}
    '''
    if type == "year":
        data = analyse.info.get_info(code,"year")['peryear']
    elif type == "quarter":
        data = analyse.info.get_info(code,"quarter")['perquarter']
    df = pd.concat([data.iloc[:,0],data.iloc[:,1:].pct_change(fill_method=None)],\
                   axis=1)
    colnum = len(df.columns)
    rownum = len(df)
    for colid in range(1,colnum):
        for rowid in range(rownum):
            if not(pd.isna(df.iloc[rowid,colid])):
                df.iloc[rowid,colid] = str(round(df.iloc[rowid,colid]*100,0)) + "%"
    return {'raw':data,'growth':df}


def get_pe(code=None,type=None):
    '''
    get_pe(code=None,type=None) code contain surfix,
            type :year,quarter(当期的数值，非累计)
    return a dictionary,  {'raw':df,'pe':df_pe} 
    '''
    if  type == "static" :
        df_pe = pd.DataFrame(data=None,columns=['report_date','pe','pe_parent'],\
                             dtype=None)
        df  = analyse.info.get_info(code=code,type='year')['peryear']
        df_pe['report_date'] = df['report_date']
        df_pe["pe"] = df["market_cap"]*100000000/df["np_parent_company_owners"]
        df_pe['pe'] = df_pe["pe"].apply(lambda x:round(x,2))
        df_pe["pe_parent"] = df["market_cap"]*100000000/\
                             df["np_parent_company_owners_parent"]
        df_pe['pe_parent']  = df_pe['pe_parent'].apply(lambda x:round(x,2))
    elif  type == "ttm":
        df_pe = pd.DataFrame(data=None,\
                             columns=['report_date','pe_ttm','pe_ttm_parent'],\
                             dtype=None)
        df  = analyse.info.get_info(code=code,type='quarter')['perquarter']
        df_pe['report_date'] = df['report_date']
        earnings_list = list(df['np_parent_company_owners'])
        earnings_list_parent = list(df['np_parent_company_owners_parent'])
        market_cap_list = list(df['market_cap'])
        for ind in range(len(df)):
            if ind == 0 :
                earnings =  earnings_list[ind]*4
                earnings_parent =  earnings_list_parent[ind]*4
            elif ind == 1 :
                earnings =  sum(earnings_list[0:ind+1])*2
                earnings_parent =  sum(earnings_list_parent[0:ind+1])*2
            elif ind == 2 :
                earnings =  sum(earnings_list[0:ind+1])*4/3
                earnings_parent =  sum(earnings_list_parent[0:ind+1])*4/3
            else :
                earnings =  sum(earnings_list[ind-3:ind+1])
                earnings_parent =  sum(earnings_list_parent[ind-3:ind+1])
            pe_ttm = market_cap_list[ind]*100000000/earnings
            pe_ttm_parent = market_cap_list[ind]*100000000/earnings_parent
            df_pe['pe_ttm'][ind] = round(pe_ttm,2)
            df_pe['pe_ttm_parent'][ind] = round(pe_ttm_parent,2)
    return  {'raw':df,'pe':df_pe}


def get_return(code=None,mtype=None):
    '''
    return a dictionary,  {'raw':df,'pe':df_pe} 
    '''
    #import numpy as np
    if mtype == 'static':
        result = pd.DataFrame(data=None,\
                        columns=['report_date','roa','roa_parent','roe','roe_parent'],\
                        dtype=None)
        df = analyse.info.get_info(code,type='year')['peryear']
        result['report_date'] = df['report_date']
        result['roa'] = df['np_parent_company_owners'] / df['total_assets']
        result['roa_parent'] = df['np_parent_company_owners_parent'] / \
                               df['total_assets_parent']
        result['roe'] = df['np_parent_company_owners'] / df['total_owner_equities']
        result['roe_parent'] = df['np_parent_company_owners_parent'] / \
                               df['total_owner_equities_parent']
    elif mtype == 'ttm':
        df = analyse.info.get_info(code,type='quarter')['perquarter']
        result = pd.DataFrame(data=None,\
                 columns=\
                 ['report_date','roa_ttm','roa_ttm_parent','roe_ttm','roe_ttm_parent'],\
                 dtype='object')
        result['report_date'] = df['report_date']
        for ind in range(len(df)):
            if ind == 0 :
                net_p = df['np_parent_company_owners'][ind]*4
                np_parent = df['np_parent_company_owners_parent'][ind]*4
                assets = df['total_assets'][ind]
                assets_parent = df['total_assets_parent'][ind]
                equities = df['total_owner_equities'][ind]
                equities_parent = df['total_owner_equities_parent'][ind]
            elif ind == 1 :
                net_p = df['np_parent_company_owners'][0:2].sum()*2
                np_parent = df['np_parent_company_owners_parent'][0:2].sum()*2
                assets = df['total_assets'][0:2].mean()
                assets_parent = df['total_assets_parent'][0:2].mean()
                equities = df['total_owner_equities'][0:2].mean()
                equities_parent = df['total_owner_equities_parent'][0:2].mean()
            elif ind == 2 :
                net_p = df['np_parent_company_owners'][0:3].sum()*4/3
                np_parent = df['np_parent_company_owners_parent'][0:3].sum()*4/3
                assets = df['total_assets'][0:3].mean()
                assets_parent = df['total_assets_parent'][0:3].mean()
                equities = df['total_owner_equities'][0:3].mean()
                equities_parent = df['total_owner_equities_parent'][0:3].mean()
            else :
                net_p = df['np_parent_company_owners'][ind-3:ind+1].sum()
                np_parent = df['np_parent_company_owners_parent'][ind-3:ind+1].sum()
                assets = df['total_assets'][ind-3:ind+1].mean()
                assets_parent = df['total_assets_parent'][ind-3:ind+1].mean()
                equities = df['total_owner_equities'][ind-3:ind+1].mean()
                equities_parent = df['total_owner_equities_parent'][ind-3:ind+1].mean()
            result['roa_ttm'][ind] = net_p/assets
            result['roa_ttm_parent'][ind] = np_parent/assets_parent
            result['roe_ttm'][ind] = net_p/equities
            result['roe_ttm_parent'][ind] = np_parent/equities_parent
        #result.iloc[:,1] = result.iloc[:,1].astype('float')
        #result.iloc[:,2] = result.iloc[:,2].astype('float')
        #result.iloc[:,3] = result.iloc[:,3].astype('float')
        #result.iloc[:,4] = result.iloc[:,4].astype('float')
    return_raw = rounddf(result)['raw']
    return_print = rounddf(result)['print']
    return  {'raw':df,'return_raw':return_raw,'return_print':return_print}
        




import os,re,time
import lxml.html
import pandas as pd
import numpy as np
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By

chrome_options = Options()
chrome_options.add_argument('--no-sandbox')
chrome_options.add_argument('--disable-dev-shm-usage')
chrome_options.add_argument("--headless")
browser = webdriver.Chrome(options=chrome_options)
wait = WebDriverWait(browser, 20)

cbondfile = os.path.join(os.environ['HOME'],'cbond.csv')

def info():
    if os.path.isfile(cbondfile) :
        df = pd.read_csv(cbondfile,sep=',',header=0)
    else : 
        df = update()
    return df


def update():
    if os.path.isfile(cbondfile) : df_bak = pd.read_csv(cbondfile,sep=',',header=0)
    try:
        fields = []
        url = "https://www.jisilu.cn/data/cbnew/#cb"
        browser.get(url)
        wait.until(lambda e: e.execute_script('return document.readyState') != "loading")
        range = browser.find_element_by_id('flex_cb')
        content = range.get_attribute('innerHTML')
        with open('/tmp/bond.html', "w", encoding='utf-8') as fh:
            fh.write(content)
        root = lxml.html.document_fromstring(open('/tmp/bond.html','r').read())
        header = root.xpath('.//thead/tr')[1].xpath('//th')
        for item in header:
             fields.append(''.join(item.text_content().split()))
        tmp_fields = []
        for idx,item in enumerate(fields):
            if idx not in [7,9,10,11,12,13,16,17,20,21,22,23,24,25,26]:
               tmp_fields.append(fields[idx])
        tmp_fields.extend(['正股名称','正股代码','转股起始日','发行规模','剩余规模'])    
        fields = tmp_fields
        data_row = root.xpath('.//tr')[2:]
        df = pd.DataFrame(columns = fields)
        for bond_info in data_row:
            every_bond = []
            for idx,item in enumerate(bond_info.xpath('.//td')):
                if idx == 0 :
                    other_info = get_element(item.text_content())
                if idx not in [7,9,10,11,12,13,16,17,20,21,22,23,24,25,26]:        
                    every_bond.append(''.join(item.text_content().split()))
            every_bond.extend([other_info['正股名称'],other_info['正股代码'],other_info['转股起始日'],other_info['发行规模'],other_info['剩余规模']])
            df.loc[len(df)] = every_bond
        df[['现价','正股价','剩余年限' ]] = pd.DataFrame(df[['现价','正股价','剩余年限']],dtype=np.float)
        df['转股价'] = df['转股价'].apply(lambda x: float(x.replace('*','')))
        df['回售触发价'] = df['回售触发价'].apply(lambda x: 0 if x == '-' else x)
        df['强赎触发价'] = df['强赎触发价'].apply(lambda x: 0 if x == '-' else x)
        df['正股涨跌'] = df['正股涨跌'].apply(lambda x:  round(float(x.replace('%',''))/100,4))
        df['涨跌幅'] = df['涨跌幅'].apply(lambda x:  round(float(x.replace('%',''))/100,4))
        curprice = df['现价']
        stockprice = df['正股价']
        conprice = df['转股价']
        premium =  (100/conprice)*stockprice/curprice - 1
        premium = premium.apply(lambda x: round(x,4))
        df['转股溢价率'] = premium
        os.remove(cbondfile)
        df.to_csv(cbondfile,index=False)
    except:
        print("update failure,please use the old data")
        df = df_bak
        del df_bak
    return df


def get_element(code):
    import lxml.html
    import urllib.request
    url="https://www.jisilu.cn/data/convert_bond_detail/{}".format(code)
    page = urllib.request.urlopen(url).read().decode("utf-8")
    root = lxml.html.document_fromstring(page)
    content = root.xpath('.//table[@class="jisilu_tcdata"]')[0]
    name_string = content.xpath('.//tr/td')[0].text_content()
    parsed = re.search("\(正股：(\S+).+?(\d+)",name_string)
    stock_name = parsed.group(1)
    stock_code = parsed.group(2)
    date = content.xpath('.//td[text()="转股起始日"]/following-sibling::td')[0].text
    scale_issue = content.xpath('.//td[text()="发行规模(亿)"]/following-sibling::td')[0].text
    scale_remain = content.xpath('.//td[text()="剩余规模(亿)"]/following-sibling::td')[0].text
    time.sleep(1)
    return  {'正股名称':stock_name ,'正股代码':stock_code,'转股起始日':date,'发行规模':scale_issue,'剩余规模':scale_remain}





#! python3
'''
calculate stock's pe at hk ipo
example:
    import analyse.data.pehkipo.getpe as getpeipo
    code = "00788"
    print(getpeipo(code))
'''
import re
import lxml.html
import urllib.request
import pandas as pd

def  getpe(code):
    currency_list = ['人民币','港元','美元','新加坡元','英镑']
    unit_list = ['千','百万']
    market_value_url = "http://www.aastocks.com/sc/stocks/market/ipo/\
                        upcomingipo/company-summary?symbol={}#info".format(code)
    income_url = "http://www.aastocks.com/sc/stocks/market/ipo/upcomingipo/\
                        profit-loss?symbol={}&s=3&o=1#info".format(code)
    market_value_page = urllib.request.urlopen(market_value_url).read().decode("utf-8")
    market_value_root = lxml.html.document_fromstring(market_value_page)
    market_value_str = market_value_root.xpath(".//table[@id='IPOInfo']//tr")[2].\
                                               text_content().replace(",","")
    market_value_base = re.findall("\d+",market_value_str)
    market_value_low = float(market_value_base[0])
    market_value_high = float(market_value_base[1])
    ipo_price = market_value_root.xpath(".//table[@id='IPOInfo']//tr")[1].\
                                         text_content().replace("招股价","").strip()
    income_page = urllib.request.urlopen(income_url).read().decode("utf-8")
    income_root = lxml.html.document_fromstring(income_page)
    currency = income_root.xpath(".//tr[.//td[text() = '货币']]//td")[1].text_content()
    unit = income_root.xpath(".//tr[.//td[text() = '单位']]//td")[1].text_content()
    if currency not in currency_list :
        print("add a new currency_list for {}".format(currency))
        return
    if currency == "港元":
        ex = 1
    elif currency == "人民币":
        ex = getex("rmb_hkd")
    elif currency == "美元":
        ex = getex("usd_hkd")
    elif currency == "新加坡元":
        ex = getex("xsd_hkd")
    if unit not in unit_list:
        print("add a new unit {} to calculate".format(unit))
        return
    if unit == '千':
        unit_value = 1000
    elif unit == '百万':
        unit_value = 1000000
    income_str = income_root.xpath(".//tr[.//td[text() = '股东应占溢利']]//td")[1].\
                                   text_content().replace(",","")
    income = ex*float(income_str)*unit_value
    code = code
    data = [code,ipo_price,income,market_value_low,market_value_high,\
            round(market_value_low/income,2),round(market_value_high/income,2)]
    col = ["代码","招股价","最近一年收益","市值_low","市值_high","pe_low","pe_high"]
    pe_df =pd.DataFrame([data],columns=col)
    return  pe_df


def getex(ex):
    ex_url = "https://www.bochk.com/whk/rates/exchangeRatesHKD/\
              exchangeRatesHKD-input.action?lang=en"
    ex_page = urllib.request.urlopen(ex_url).read().decode("utf-8")
    ex_root = lxml.html.document_fromstring(ex_page)
    if ex == "rmb_hkd":
        exrate = ex_root.xpath(".//tr[.//td[text() = 'CNY']]/td")[1].\
                                text_content().strip()
        exrate =  round(float(exrate),4)
    elif ex == "usd_hkd":
        exrate = ex_root.xpath(".//tr[.//td[text() = 'USD']]/td")[1].\
                                text_content().strip()
        exrate =  round(float(exrate),4)
    elif ex == "sgd_hkd":
        exrate = ex_root.xpath(".//tr[.//td[text() = 'SGD']]/td")[1].\
                                text_content().strip()
        exrate =  round(float(exrate),4)
    return exrate



#!/usr/bin/python3  
'''
从通达信下载数据
example:
    python3 writedata.py
    or
    import analyse.data
    analyse.data.writedata.main()
'''
from jqdatasdk import *
import jqdatasdk
from  os import listdir as ldir
import os ,shutil ,io
import pymysql
from sqlalchemy import create_engine
from sqlalchemy.sql import text
import pandas as pd
import struct
import re,time,datetime
import http.cookiejar
import urllib.request
import logging
import smtplib, ssl
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
import  analyse.cncode as cncode
import  analyse.hkcode as hkcode
import subprocess


base_dir = '/tmp' 
logging_file = os.path.join(base_dir,'writedata.log')
logging.basicConfig(filename=logging_file,level=logging.INFO)

user = 'root'
mysql_ip = '127.0.0.1'
mysql_pass = os.environ['mysql_pass']
acc_jq = os.environ['acc_jq']
passwd_jq = os.environ['passwd_jq']
email126 = os.environ['email126']
pass126 = os.environ['pass126']

hkengine = create_engine("mysql+pymysql://{}:{}@{}:3306/hksec?charset=utf8".\
                         format(user,mysql_pass,mysql_ip))
szengine = create_engine("mysql+pymysql://{}:{}@{}:3306/szsec?charset=utf8".\
                         format(user,mysql_pass,mysql_ip))
shengine = create_engine("mysql+pymysql://{}:{}@{}:3306/shsec?charset=utf8".\
                         format(user,mysql_pass,mysql_ip))
hkdir = os.path.join(base_dir,'hklday')
szdir = os.path.join(base_dir,'szlday')
shdir = os.path.join(base_dir,'shlday')
hkfile = os.path.join(base_dir,'hklday.zip')
szfile = os.path.join(base_dir,'szlday.zip')
shfile = os.path.join(base_dir,'shlday.zip')

mproperty = {'hk':{'engine':hkengine,'mdir':hkdir},
             'sz':{'engine':szengine,'mdir':szdir},
             'sh':{'engine':shengine,'mdir':shdir}}

unpack_rule = {'hk':'ifffffii','sz':'iiiiifii','sh':'iiiiifii'}

def get_latest(target):
    ''' get_all_trade_days create all trades day in a year,getlast get the 
    latest trade day forword'''
    xday = get_all_trade_days()
    latest = target + datetime.timedelta(-1)
    while (latest not in xday):
        latest = latest + datetime.timedelta(-1)
    return latest


def send_email(subject="title",content="info"):
    '''call 126 mail and send mail to 126 itself'''
    port = 465
    smtp_server = 'smtp.126.com'
    sender_email = email126
    receiver_email = email126
    password = pass126
    message = MIMEMultipart('alternative')
    message['Subject'] = subject
    message['From'] = sender_email
    message['To'] = receiver_email
    text = content
    part = MIMEText(text, 'plain')
    message.attach(part)
    context = ssl.create_default_context()
    with smtplib.SMTP_SSL('smtp.126.com', port, context=context) as server:
        server.login(sender_email, password)
        server.sendmail(sender_email, receiver_email, message.as_string())


def write_quote_table(code,dfile,market):
    '''parse data from file and write into table quote
    return quote data from every stock in dataframe form
    '''
    try:
        data = open(dfile,'rb')
        dayline = data.read(32)
        alldayline = []
        while  dayline:
            tmpdata = list(struct.unpack(unpack_rule[market],dayline))
            dataday = [code]
            if( code == '510050' and market == 'sz'):
                dataday.append(str(tmpdata[0]))
                dataday.append(tmpdata[1]/1000)
                dataday.append(tmpdata[2]/1000)
                dataday.append(tmpdata[3]/1000)
                dataday.append(tmpdata[4]/1000)
                dataday.append(round(tmpdata[5],2))
                dataday.append(tmpdata[6])
                alldayline.append(dataday)
            elif( code == '510050' and market == 'sh'):
                dataday.append(str(tmpdata[0]))
                dataday.append(tmpdata[1]/1000)
                dataday.append(tmpdata[2]/1000)
                dataday.append(tmpdata[3]/1000)
                dataday.append(tmpdata[4]/1000)
                dataday.append(round(tmpdata[5],2))
                dataday.append(tmpdata[6])
                alldayline.append(dataday)
            elif( len( str(tmpdata[0]) )  == 8 and market == 'hk'):
                dataday.append(str(tmpdata[0]))
                dataday.append(round(tmpdata[1],2))
                dataday.append(round(tmpdata[2],2))
                dataday.append(round(tmpdata[3],2))
                dataday.append(round(tmpdata[4],2))
                dataday.append(round(tmpdata[5],2))
                dataday.append(tmpdata[6]*100)
                alldayline.append(dataday)
            elif( len( str(tmpdata[0]) )  == 8 and (market == 'sz' ) ):
                dataday.append(str(tmpdata[0]))
                dataday.append(tmpdata[1]/100)
                dataday.append(tmpdata[2]/100)
                dataday.append(tmpdata[3]/100)
                dataday.append(tmpdata[4]/100)
                dataday.append(round(tmpdata[5],2))
                dataday.append(tmpdata[6])
                alldayline.append(dataday)
            elif( len( str(tmpdata[0]) )  == 8 and (market == 'sh') ):
                dataday.append(str(tmpdata[0]))
                dataday.append(tmpdata[1]/100)
                dataday.append(tmpdata[2]/100)
                dataday.append(tmpdata[3]/100)
                dataday.append(tmpdata[4]/100)
                dataday.append(round(tmpdata[5],2))
                dataday.append(tmpdata[6])
                alldayline.append(dataday)
            else:
                logging.info('parse stock file {} , failure , \
                              contain some 00 as date'.format(dfile))
                return None
            dayline = data.read(32)
    except:
        logging.info('parse error unknown for  {} '.format(dfile))
        return None
    fields = ['code','date','open','high','low','close','amount','volume']
    df = pd.DataFrame(alldayline, columns= fields)
    df.to_sql(name = 'quote', con = mproperty[market]['engine'], \
              if_exists='append', index=False)
    return df


def get_cur_vix(code,data,market):
    '''get the current vix for every stock from recent trend
    and write it into curvix table 
    return stock's current vix in array 
    
    '''
    price = data['close']
    date = data['date'].map(lambda x:str(x))
    number = len(data)
    lmax_min = []
    if number<21 :
        logging.info("the stock {}  's data is too small  ".format(code))
        return None
    end_id = number - 1
    id=10
    while id<= end_id-11:
        flag1 = max(price[id-10:id])
        flag2 = min(price[id+1:id+11])
        flag3 = min(price[id-10:id])
        flag4 = max(price[id+1:id+11])
        if ( price[id] >= flag1 and price[id] >= flag4 ):
            lmax_min.append([date[id],price[id],'lmax'])
            id = id +10
        elif( price[id] <= flag2 and price[id] <= flag3 ):
            lmax_min.append([date[id],price[id],'lmin'])
            id = id +10
        else:
            id = id +1
    end_index = len(data) - 1
    if lmax_min:
        if  lmax_min[-1][1] >0:
            start_index = data[data['date'] == lmax_min[-1][0]].index.values[0]
            cur_time_scope = end_index - start_index + 1
            if(lmax_min[-1][1] >= list(price)[-1]):
                vix_range = -round( 1- list(price)[-1] / lmax_min[-1][1] , 4)
            else:
                vix_range = round( list(price)[-1]/lmax_min[-1][1] -1 , 4)
        else:
            logging.info("max or min value is zero,can't calculate the vix for  {}  "\
                          .format(code))
            return None
    else:
        logging.info("no max or min value for {}  because of short time period "\
                      .format(code))
        return None
    maxdate = list(date)[-1]
    mdata = [code,cur_time_scope,vix_range,maxdate]
    return  mdata


def write_hsi():
    '''get hsi data from yahoo and write it into mysql'''
    code = 'hsi'
    start=0
    end=int(time.time())
    symbol='^hsi'
    target1 = "https://finance.yahoo.com/quote/{}/?p={}".format(symbol,symbol)
    fname_cookie = os.path.join(base_dir,'cookies.txt')
    cookie = http.cookiejar.MozillaCookieJar(fname_cookie)
    handler = urllib.request.HTTPCookieProcessor(cookie)
    opener = urllib.request.build_opener(handler)
    response1 = opener.open(target1).read().decode('utf-8')
    cookie.save()
    crumb = re.findall('"CrumbStore":{"crumb":"(.+?)"}', response1)[0]
    cookie.load(fname_cookie)
    handler = urllib.request.HTTPCookieProcessor(cookie)
    opener = urllib.request.build_opener(handler)
    target2 = "https://query1.finance.yahoo.com/v7/finance/download/{0}?period1={1}\
               &period2={2}&interval=1d&events=history&crumb={3}".\
               format(symbol,start,end,crumb)
    response2 = opener.open(target2).read().decode('utf-8')
    df = pd.read_csv(io.StringIO(response2),sep=',')
    tmpdf = df.dropna(axis=0,how='any')
    tmpdf.columns = ['date','open','high','low','close','amount','volume']
    newdf = pd.DataFrame(columns = ['code','date','open','high','low',\
                                    'close','amount','volume'] )
    for id in range(0,len(tmpdf)):
        newdf.loc[id] = [code,tmpdf.iloc[id]['date'],round(tmpdf.iloc[id]['open'],2),  \
                     round(tmpdf.iloc[id]['high'],2),round(tmpdf.iloc[id]['low'],2),   \
                     round(tmpdf.iloc[id]['close'],2),0,int(tmpdf.iloc[id]['volume'])]
    newdf.to_sql('quote', con=hkengine, index=False, if_exists='append')
    os.remove(fname_cookie)
    return newdf


def parse_code(dfile,market):
    '''parse code from dfile'''
    if market == 'hk' :
        code = re.search('\d{5}',dfile).group()
    else:
        code = re.search('\d{6}',dfile).group()
    return  code

def prework():
    '''
    download raw data from tdx and create database table
    '''
    cmd = "mysqldump  -uroot -pi{} --all-databases > {}/alldatabaseback.sql".\
                 format(mysql_pass,os.environ['HOME'])
    os.system(cmd)
    if(not ( os.path.exists(hkfile) and os.path.exists(szfile) and \
             os.path.exists(shfile))):
        downloadfile = os.path.join(os.path.dirname(__file__),'download.sh')
        os.system("/bin/bash {}".format(downloadfile))
    try:
        sqlfile = os.path.join(os.path.dirname(__file__),'init.sql')
        os.system("mysql -h localhost -uroot -p{} < {}".format(mysql_pass,sqlfile))
    except:
        send_email(subject='database',content='something wrong \
                   with your database setting')
        exit()


def write_code_table():
    '''
    write code into hksec,szsec,shsec 
    return stocks code 
    '''
    sz_stock,sh_stock = cncode.write_code()
    hk_stock = hkcode.write_code('all')
    stocks = {'hk':hk_stock,'sz':sz_stock,'sh':sh_stock}
    return stocks

def main():
    starttime = datetime.datetime.now()
    prework()
    stocks = write_code_table()
    for market in mproperty:
        dfns = ldir(mproperty[market]["mdir"])
        vix_data = []
        for dfn in dfns:
            #增加判断，如果dfn在代码表内，请执行解析
            dfile = os.path.join(mproperty[market]["mdir"],dfn)
            try:
                code = parse_code(dfile,market)
            except:
                logging.info("can't get code ,fail {}".format(dfile))
                continue
            if(code in list(stocks[market])):
                data = write_quote_table(code,dfile,market)
                if data is not None :
                    tmp_vix_data = get_cur_vix(code,data,market)
                    if tmp_vix_data is not None:
                        vix_data.append(tmp_vix_data)
        df_vix = pd.DataFrame(vix_data,columns = ['code','timescope','vix','maxdate'])
        df_vix.to_sql('curvix',con= mproperty[market]['engine'],if_exists='append',\
                      index=False)        
    time.sleep(10)
    content_hsi = ''
    try:
        write_hsi()
    except:
        logging.info('can not download hsi ')
        content_hsi = "something wrong with hsi data,can't download it\n"
    #remove all temp data
    shutil.rmtree(hkdir)
    shutil.rmtree(szdir)
    shutil.rmtree(shdir)
    os.remove(hkfile)
    os.remove(szfile)
    os.remove(shfile)
    os.remove(os.path.join(os.environ['HOME'] , 'alldatabaseback.sql'))
    send_email(subject = 'get data',content= content_hsi + 'all data downloaded ,\
               write into database and get curvix ')
    endtime = datetime.datetime.now()
    mytime = round((endtime -starttime).seconds/60,2)
    logging.info("the used  time for the whole process:download and write into \
                  database and get current vix table  is {} minutes.".format(mytime))

if  __name__ == "__main__":
    main()



__all__ = ['cncode','hkcode','data','info','vix','ana','cbond']
from analyse  import  *
import analyse
#import os
#import numpy as np
#import pandas as pd
#import datetime
#from jqdatasdk import *



#acc_jq = os.environ["acc_jq"]
#passwd_jq = os.environ["passwd_jq"]

#auth(acc_jq,passwd_jq)

__doc__ ='''
analyse文件夹放置/usr/local/lib/python3.5/dist-packages
聚宽使用后缀 .XSHG .XSHE ,其他地方的数据不用后缀；
通达信没有后缀，香港使用5位数字。
本地完成的数据库szsec,shsec,hksec都不使用后缀，
本地数据库和聚宽混合使用时，需要调整股票代码。
金融类公司和非金融类公司的财务不同，调用时，需要进行专门处理。
财务报表分别是合并报表（不含后缀），母公司报表（含后缀parent）。
cncode,hkcode的返回值，不包含后缀；pehkipo(code)参数中的code,不包含后缀；
其他函数中的code 需要使用聚宽的后缀。
info.get_info(code,type),这里的type取值year, quarter；
返回值，peryear,perquarter,收益是当季的收益，不是累计收益。
ana包中的函数输出结果分别有合并公司计算值（不包含后缀），母公司计算值（含后缀parent)。
ana.get_pe(code,type),这里的type取值static,ttm,分别对应info.get_info中的year,quarter；
ana.get_growth(code,type),这里的type取值static,ttm,分别对应info.get_info中的year,quarter；
ana.get_return(code,type),这里的type取值static,ttm,分别对应info.get_info中的year,quarter；

获得转债数据
$HOME/cbond.csv文件不可以删除
import analyse
df = analyse.cbond.info()
转债数据更新
df = analyse.cbond.update()
'''







import os
import numpy as np
import pandas as pd
import datetime
from jqdatasdk import *
acc_jq = os.environ["acc_jq"]
passwd_jq = os.environ["passwd_jq"]
auth(acc_jq,passwd_jq)

def get_growth(code):
    q = query(finance.STK_COMPANY_INFO.industry_id).filter(finance.STK_COMPANY_INFO.code==code)
    industry = finance.run_query(q).iloc[0,0]
    financeIndustry = ["J66","J67","J68","J69"]
    if ( industry in  financeIndustry ) :
        q1 = query(finance.FINANCE_INCOME_STATEMENT.report_date,
                   finance.FINANCE_INCOME_STATEMENT.operating_revenue,
                   finance.FINANCE_INCOME_STATEMENT.net_profit,
                   finance.FINANCE_INCOME_STATEMENT.np_parent_company_owners
                   ).filter(finance.FINANCE_INCOME_STATEMENT.code == code,
                   finance.FINANCE_INCOME_STATEMENT.end_date.ilike('_____12-31'), 
                   finance.FINANCE_INCOME_STATEMENT.report_type==0)
        df1 = finance.run_query(q1)
        q2 = query(finance.FINANCE_INCOME_STATEMENT_PARENT.report_date,
                   finance.FINANCE_INCOME_STATEMENT_PARENT.operating_revenue,
                   finance.FINANCE_INCOME_STATEMENT_PARENT.net_profit,
                   finance.FINANCE_INCOME_STATEMENT_PARENT.np_parent_company_owners
                   ).filter(finance.FINANCE_INCOME_STATEMENT_PARENT.code == code ,
                   finance.FINANCE_INCOME_STATEMENT_PARENT.end_date.ilike('_____12-31'), 
                   finance.FINANCE_INCOME_STATEMENT_PARENT.report_type==0)
        df2 = finance.run_query(q2)
        df2.rename(columns={'operating_revenue':'operating_revenue_parent', 
               'np_parent_company_owners':'np_parent_company_owners_parent'}, inplace = True)
    else :
        q1 = query(finance.STK_INCOME_STATEMENT.report_date,
                   finance.STK_INCOME_STATEMENT.total_operating_revenue,
                   finance.STK_INCOME_STATEMENT.net_profit,
                   finance.STK_INCOME_STATEMENT.np_parent_company_owners
                   ).filter(finance.STK_INCOME_STATEMENT.code == code ,
                   finance.STK_INCOME_STATEMENT.end_date.ilike('_____12-31'), 
                   finance.STK_INCOME_STATEMENT.report_type==0)
        df1 = finance.run_query(q1)
        q2 = query(finance.STK_INCOME_STATEMENT_PARENT.report_date,
                   finance.STK_INCOME_STATEMENT_PARENT.total_operating_revenue,
                   finance.STK_INCOME_STATEMENT_PARENT.net_profit,
                   finance.STK_INCOME_STATEMENT_PARENT.np_parent_company_owners
                   ).filter(finance.STK_INCOME_STATEMENT_PARENT.code == code ,
                   finance.STK_INCOME_STATEMENT_PARENT.end_date.ilike('_____12-31'), 
                   finance.STK_INCOME_STATEMENT_PARENT.report_type==0)
        df2 = finance.run_query(q2)
        df2.rename(columns={'total_operating_revenue':'total_operating_revenue_parent', 
               'net_profit':'net_profit_parent',
               'np_parent_company_owners':'np_parent_company_owners_parent'}, inplace = True)
    df = df1.join(df2.set_index('report_date'),on='report_date',how='outer')
    back = df 
    df = pd.concat([df.iloc[:,0],df.iloc[:,1:].pct_change(fill_method=None)],axis=1)
    colnum = len(df.columns)
    rownum = len(df)
    for colid in range(1,colnum):
        for rowid in range(rownum):
            if not(pd.isna(df.iloc[rowid,colid])):
                df.iloc[rowid,colid] = str(round(df.iloc[rowid,colid]*100,0)) + "%"
    return  {"primitive":back,"growth":df}


def get_pe_year(code):
    data_growth = get_growth(code)
    date = data_growth["primitive"]["report_date"]
    market_cap = pd.DataFrame(data=None,index=range(len(date)) ,columns=['report_date','market_cap'], dtype=None)
    for i in range(len(date)):
        q = query(valuation.market_cap).filter(valuation.code == code)
        y = get_fundamentals(q,date=date[i])
        if len(y) == 0 :
            market_cap.iloc[i,1] = np.nan
        else:
            market_cap.iloc[i,1] = y['market_cap'][0]
        market_cap.iloc[i,0] = date[i]
    raw = data_growth['primitive']
    df = raw.join(market_cap.set_index('report_date'),on='report_date',how='outer')
    df["pe"] = df["market_cap"]*100000000/df["np_parent_company_owners"]
    df["pe"] = df["pe"].apply(lambda x:round(x,2))
    df["pe_parent"] = df["market_cap"]*100000000/df["np_parent_company_owners_parent"]
    df["pe_parent"] = df["pe_parent"].apply(lambda x:round(x,2))
    return df


def get_pe_ttm(code):
    '''
       没有使用综合收益
    '''
    result = pd.DataFrame(data=None,index=range(1) ,columns=['report_date','pe_ttm','pe_ttm_parent'], dtype=None)
    today = datetime.date.today().strftime("%Y-%m-%d")    
    query_market_cap = query(valuation.market_cap).filter(valuation.code == code) 
    market_cap = get_fundamentals(query_market_cap)
    data = get_growth(code)['primitive']
    report_date = list(data['report_date'])[-1]

    query_ttm = query(valuation.pe_ratio).filter(valuation.code == code)
    pe_ttm = round(get_fundamentals(query_ttm).iloc[0,0],2)

    query_attr = query(finance.STK_COMPANY_INFO.industry_id).filter(finance.STK_COMPANY_INFO.code==code)
    industry = finance.run_query(query_attr).iloc[0,0]
    financeIndustry = ["J66","J67","J68","J69"]
    if ( industry in  financeIndustry ) :
        query_profit_parent = query(finance.FINANCE_INCOME_STATEMENT_PARENT.start_date,
                        finance.FINANCE_INCOME_STATEMENT_PARENT.end_date,
                        finance.FINANCE_INCOME_STATEMENT_PARENT.np_parent_company_owners).filter(
                        finance.FINANCE_INCOME_STATEMENT_PARENT.code == code,
                        finance.FINANCE_INCOME_STATEMENT_PARENT.report_date < today,
                        finance.FINANCE_INCOME_STATEMENT_PARENT.report_type== 0)
    else :
        query_profit_parent = query(finance.STK_INCOME_STATEMENT_PARENT.start_date,
                        finance.STK_INCOME_STATEMENT_PARENT.end_date,
                        finance.STK_INCOME_STATEMENT_PARENT.np_parent_company_owners).filter(
                        finance.STK_INCOME_STATEMENT_PARENT.code == code,
                        finance.STK_INCOME_STATEMENT_PARENT.report_date < today,
                        finance.STK_INCOME_STATEMENT_PARENT.report_type== 0)
    df = finance.run_query(query_profit_parent)
    if len(df) >= 7 :
        raw = df.iloc[-7:,]
        raw.index = range(0,7)
        flag = [0,0,0,0]
        date_list = raw['end_date'].apply(lambda x:x.strftime("%Y-%m-%d"))[-4:]
        earnings =  list(raw['np_parent_company_owners'])
        for id,item in enumerate(list(date_list)):
            if item[5:]  == '03-31':
                flag[id] = "q1"
            elif  item[5:]  == '06-30':
                flag[id] = "q2"
            elif  item[5:]  == '09-30':
                flag[id] = "q3"
            elif  item[5:]  == '12-31':
                flag[id] = "q4"
        if flag == ['q1','q2','q3','q4']:
            earning = earnings[-1]
        elif flag == ['q2','q3','q4','q1']:
           earning = earnings[-1] + earnings[-2] - earnings[-5]
        elif flag == ['q3','q4','q1','q2']:
           earning = earnings[-1] + earnings[-3] - earnings[-5]
        elif flag == ['q4','q1','q2','q3']:
           earning = earnings[-1] + earnings[-4] - earnings[-5]
        pe_ttm_parent = market_cap*100000000/earning
        result['pe_ttm_parent'] = round(pe_ttm_parent,2)
    else:
        result['pe_ttm_parent'] = np.nan 
    result['report_date'] = report_date
    result['pe_ttm'] = pe_ttm
    return result


def get_roe(code):
    result = pd.DataFrame(data=None,index=range(1) ,columns=['report_date','pe_ttm','pe_ttm_parent'], dtype=None)
    today = datetime.date.today().strftime("%Y-%m-%d")    
    query_attr = query(finance.STK_COMPANY_INFO.industry_id).filter(finance.STK_COMPANY_INFO.code == code)
    industry = finance.run_query(query_attr).iloc[0,0]
    financeIndustry = ["J66","J67","J68","J69"]
    if ( industry in  financeIndustry ) :
        query_profit = query(finance.FINANCE_INCOME_STATEMENT.report_date,
                      finance.FINANCE_INCOME_STATEMENT.np_parent_company_owners,
                      ).filter(
                      finance.FINANCE_INCOME_STATEMENT.code == code,
                      finance.FINANCE_INCOME_STATEMENT.report_type== 0)
        query_asset = query(finance.FINANCE_BALANCE_SHEET.report_date,
                            finance.FINANCE_BALANCE_SHEET.total_assets,
                            finance.FINANCE_BALANCE_SHEET.total_owner_equities
                            ).filter(
                            finance.FINANCE_BALANCE_SHEET.code == code,
                            finance.FINANCE_BALANCE_SHEET.report_type== 0)
        query_profit_parent = query(finance.FINANCE_INCOME_STATEMENT_PARENT.report_date,
                             finance.FINANCE_INCOME_STATEMENT_PARENT.np_parent_company_owners,
                             ).filter(
                             finance.FINANCE_INCOME_STATEMENT_PARENT.code == code,
                             finance.FINANCE_INCOME_STATEMENT_PARENT.report_type== 0)
        query_asset_parent = query(finance.FINANCE_BALANCE_SHEET_PARENT.report_date,
                                   finance.FINANCE_BALANCE_SHEET_PARENT.total_assets,
                                   finance.FINANCE_BALANCE_SHEET_PARENT.total_owner_equities
                                   ).filter(
                                   finance.FINANCE_BALANCE_SHEET_PARENT.code == code,
                                   finance.FINANCE_BALANCE_SHEET_PARENT.report_type== 0)
    else :
        query_profit = query(finance.STK_INCOME_STATEMENT.report_date,
                      finance.STK_INCOME_STATEMENT.np_parent_company_owners,
                      ).filter(
                      finance.STK_INCOME_STATEMENT.code == code,
                      finance.STK_INCOME_STATEMENT.report_type== 0)
        query_asset = query(finance.STK_BALANCE_SHEET.report_date,
                      finance.STK_BALANCE_SHEET.total_assets,
                      finance.STK_BALANCE_SHEET.total_owner_equities
                      ).filter(
                      finance.STK_BALANCE_SHEET.code == code,
                      finance.STK_BALANCE_SHEET.report_type== 0)
        query_profit_parent = query(finance.STK_INCOME_STATEMENT_PARENT.report_date,
                             finance.STK_INCOME_STATEMENT_PARENT.np_parent_company_owners,
                             ).filter(
                             finance.STK_INCOME_STATEMENT_PARENT.code == code,
                             finance.STK_INCOME_STATEMENT_PARENT.report_type== 0)
        query_asset_parent = query(finance.STK_BALANCE_SHEET_PARENT.report_date,
                                   finance.STK_BALANCE_SHEET_PARENT.total_assets,
                                   finance.STK_BALANCE_SHEET_PARENT.total_owner_equities
                                   ).filter(
                                   finance.STK_BALANCE_SHEET_PARENT.code == code,
                                   finance.STK_BALANCE_SHEET_PARENT.report_type== 0)
    df1 = finance.run_query(query_profit)
    df2 = finance.run_query(query_asset)
    df3 = finance.run_query(query_profit_parent)
    df4 = finance.run_query(query_asset_parent)
    df = df1.join(df2.set_index('report_date'),on='report_date',how='outer')
    df['roa'] = df['np_parent_company_owners'] / df['total_assets']
    df['roa'] = df['roa'].apply(lambda x: str(round(x*100,2))+'%' if np.isnan(x) == False else x)
    df['roe'] = df['np_parent_company_owners'] / df['total_owner_equities']
    df['roe'] = df['roe'].apply(lambda x: str(round(x*100,2))+'%' if np.isnan(x) == False else x)
    return df  



# -*- coding: utf-8 -*-
'''
get and write hkcode into database
'''
import time,os,sys
import pandas as pd
import re 
from sqlalchemy import create_engine
import argparse

mysql_pass = os.environ['mysql_pass']
mysql_ip = '127.0.0.1'
table_all = 'allsec'
table_anchor = 'anchorsec'
table_other = 'othersec'

def get_time():
    '''
    get time for retrieve url ,minusNumber

    '''
    today = time.localtime()
    dayOfWeek = time.strftime('%u',today)
    if(dayOfWeek == '1'):
        minusNumber = 3
    elif(dayOfWeek == '7'):
        minusNumber = 2
    else:
        minusNumber = 1
    stamptime_getData = int(time.strftime('%s',today)) - minusNumber * 24 * 60 *60
    iday = time.strftime('%d',time.localtime(stamptime_getData))
    if(iday[0] == '0'): iday = iday[1]
    imon = time.strftime('%b',time.localtime(stamptime_getData))
    iyear = time.strftime('%y',time.localtime(stamptime_getData))
    return [iday,imon,iyear]


def con5code(code):
    '''
    hkcode contain 5 digitals
    series converter ,map function
    '''
    if(code[0] != '8'):
        code = '0' + code[0:4]
    else:
        code = code[0:5]
    return code

def get_code(type):
    '''
    all:all securities in hk,table name allsec
    anchor:indexed and stock option,table name anchorsec
    other: customized,table name othersec
    '''
    if (type == 'all'):
        url_all1 = 'https://sc.hkex.com.hk/TuniS/www.hkex.com.hk/chi/services'
        url_all2 = '/trading/securities/securitieslists/ListOfSecurities_c.xlsx'
        url_all = url_all1 + url_all2
        os.system("wget  --user-agent='Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.3) Gecko/2008092416 Firefox/3.0.' {} -O  /tmp/all.xlsx ".format(url_all))
        data_all = pd.read_excel(io="/tmp/all.xlsx",skiprows=2,dtype=str)
        df_all = pd.DataFrame(data_all['股份代號'])
        df_all.columns = ['code']
        return df_all
    elif(type == 'anchor'):
        codes_anchor = set()
        #filter 
        itime=get_time()
        #hsi 50
        url_hsi = "https://www.hsi.com.hk/static/uploads/contents/en/indexes/\
                   report/hsi/con_{0}{1}{2}.csv".format(itime[0],itime[1],itime[2])
        os.system("wget {} -O /tmp/anchor.csv".format(url_hsi))
        data_hsi =  pd.read_csv(io='/tmp/anchor.csv',dtype=str,encoding = 'UTF-16',\
                                skiprows=1,sep='\t')
        codes_hsi = data_hsi['Stock Code']
        #hsi中型指数
        url_hsci = "https://www.hsi.com.hk/static/uploads/contents/en/indexes/\
                    report/hsci/con_{0}{1}{2}.csv".format(itime[0],itime[1],itime[2])
        os.system("wget {} -O /tmp/hsci.csv".format(url_hsci))
        data_hsci =  pd.read_csv(io='/tmp/hsci.csv',dtype=str,encoding = 'UTF-16',\
                                 skiprows=1,sep='\t')
        filter = data_hsci['Index'] == 'Hang Seng Composite MidCap Index \
                                        恒生綜合中型股指數' 
        codes_hsci = data_hsci[filter]['Stock Code']
        codes_anchor = codes_hsi.append(codes_hsci)
        codes_anchor = codes_anchor.map(con5code)
        frame = {'code':codes_anchor}
        # serires to dictionary to dataframe 
        df_anchor = pd.DataFrame(frame)
        df_anchor.index = range(len(df_anchor))
        return df_anchor
    elif(type == 'other'):
        codes_other = []
        while True:
            try:
                code = input('please input what stock you want with five digital,\
                              enter ctrl+d to finish the input: ')
                if len(code) == 5:
                    codes_other.append(code)
                else:
                    print('please input what stock you want with five digital')
                    continue
            except EOFError:
                break
        df = pd.DataFrame(codes_other,columns = ['code'])
        return  df

def convert_unit(value):
    tmpvalue = value.replace(',','')
    tmpvalue = int(tmpvalue)
    return tmpvalue 


def write_code(type):
    '''
    write codes into database,all allsec;anchor anchorsec;other othersec
    write_code('all')  write_code('anchor')  write_code('other')
    '''
    conn = create_engine("mysql+pymysql://root:{}@{}:3306/hksec?charset=utf8".\
                          format(mysql_pass,mysql_ip))
    if(type == 'all'):
        url_all = 'https://sc.hkex.com.hk/TuniS/www.hkex.com.hk/chi/services/\
                   trading/securities/securitieslists/ListOfSecurities_c.xlsx'
        data_all = pd.read_excel(url_all,skiprows=2,dtype=str)
        fields = ['code','name','type','unit']
        data = data_all[['股份代號','股份名稱','分類','買賣單位']]
        data.columns = fields
        xdata = data.copy()
        xdata['unit'] = xdata['unit'].map(convert_unit)
        table = eval("table_{}".format(type))
        xdata.to_sql(table, conn,if_exists='replace', index=False)
        return  xdata['code']
    elif(type == 'anchor' or type == 'other'):
        data = get_code(type)
        table = eval('table_{}'.format(type))
        if (type == 'anchor'):
            data.to_sql(table, conn,if_exists='replace', index=False)
        else:
            data.to_sql(table, conn,if_exists='append', index=False)
        return  data['code']



#!/usr/bin/env python3
# coding=utf-8

from setuptools import setup, find_packages

setup(
    name='myanalysestock',
    version=0.1,
    author='pengsir',
    author_email='qq@qq.com',
    maintainer='pengsir',
    maintainer_email='qq@qq.com',
    license='BSD License',
    packages=find_packages(),
    description="analyse stock",
    platforms=["all"],
    url='https://pypi.org/project/myanalysestock/',
    classifiers=[
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries'
    ],
)
